# -*- coding: utf-8 -*-

import datetime
import json
import time
import re

import click
import dateparser

from em_tasks.utils.price_calculator import PriceCalculator

from em_celery import logger, get_config, init_db, get_product_service


@click.command('Filter ebay uploable products.')
@click.option('-m', '--marketplace', type=str, default='us')
@click.option('-t', '--ttl', type=int, default=7, help='Product information alive days, default is 7.')
@click.option('-l', '--location_id', required=True, type=str, help='Location ID.')
@click.option('-o', '--output_path', type=str, required=True)
@click.argument('products_path')
def format_products(products_path, output_path, location_id, marketplace='us', ttl=7):
  init_db()
  product_service = get_product_service()

  price_rules = {
    'roi': 0.3,
    'ad_cost': 4.5,
    'transfer_cost': 0
  }

  marketplace = marketplace.lower()
  cfg = get_config()
  price_rules_cfg = cfg.get('price.rules.ebay_{}'.format(marketplace), dict())
  price_rules.update(price_rules_cfg)
  price_calculator = PriceCalculator(price_rules)

  ebay_formatter = EbayProductFormatter(products_path, output_path, product_service, price_calculator, location_id, marketplace, ttl)
  ebay_formatter.run()


class EbayProductFormatter():
  def __init__(self, products_path, output_path, product_service, price_calculator, location_id, marketplace, ttl=7):
    self.products_path = products_path
    self.output_path = output_path
    self.product_service = product_service
    self.price_calc = price_calculator
    self.marketplace = marketplace
    self.location_id = location_id
    self.indice_name = 'ebay_{}_products'.format(marketplace.lower())
    self.output_fh = open(self.output_path, 'w+', encoding='utf-8', errors='ignore')
    today = datetime.date.today()
    self.expire_date = today - datetime.timedelta(days=ttl) 
    self.ebay_pattern = re.compile(re.escape('ebay'), re.IGNORECASE)

  def run(self):
    batch_size = 500
    product_ids = []
    try:
      with open(self.products_path, encoding='utf-8', errors='ignore') as fh:
        for line in fh:
          s = line.strip()
          if not s:
            continue

          product_ids.append(s)
          if len(product_ids) < batch_size:
            continue

          self.process_products(product_ids)
          product_ids = []

      if product_ids:
        self.process_products(product_ids)
        product_ids = []
    finally:
      self.output_fh.close()

  def process_products(self, product_ids):
    products = False
    max_retries = 3
    while max_retries > 0:
      try:
        products = self.product_service.search_products(self.indice_name, product_ids)
        if not isinstance(products, dict):
          logger.info('[ProductService] Temporary unavailable! Wait for 10 seconds to retry.')
          max_retries -= 1
          time.sleep(10)
          continue

        break
      except Exception as e:
        logger.exception(e)
        max_retries -= 1
        time.sleep(3)

    if not products:
      return

    for pid, product in products.items():
      has_only_default_variant = product.get('has_only_default_variant', True)
      if not has_only_default_variant:
        continue

      update_date_s = product.get('date', product.get('time', None))
      if not update_date_s:
        continue

      try:
        update_date = dateparser.parse(update_date_s).date()
        if update_date < self.expire_date:
          continue
      except:
        continue

      shipping_days_max = product.get('shipping_days_max', None)
      if not shipping_days_max or shipping_days_max > 14:
        logger.debug('[ShippingTimeInvalid] %s', product['product_id'])
        continue

      seller_rating = product.get('seller_rating', 0)
      if not seller_rating >= 98:
        logger.debug(
          '[SellerRatingInvalid] ProductId: %s, SellerRating: %s',
          product['product_id'], seller_rating)
        continue

      prod = self.format_product(product)
      if not prod:
        continue

      self.output_fh.write("{}\n".format(json.dumps(prod)))

  def format_product(self, product):
    src_product_price = product.get('price', None)
    if not src_product_price:
      return

    src_shipping_fee = product.get('shipping_fee', 0) or 0
    currency = product.get('currency', None) or 'USD'
    src_offer = {'price': src_product_price + src_shipping_fee, 'currency': currency}
    offer = self.price_calc.calc_offer(src_offer)
    if not offer or not offer['quantity']:
      return

    product['price'] = offer['price']
    product['quantity'] = offer['quantity']
    product['sku'] = product['sku'].replace('Ebay_US', 'X02').replace('Ebay', 'X02').replace('_', '-')
    product['description'] = self.ebay_pattern.sub('', product['description'])
    product['location_id'] = self.location_id
    product.pop('seller_id', None)

    categories = product['categories']
    if categories:
      cats = []
      parts = categories.split('>')
      for part in parts:
        pl = part.lower()
        if pl.find('ebay') != -1 or pl.find('see more') != -1:
          continue

        cats.append(part)

      product['categories'] = '>'.join(cats)

    return product


if __name__ == '__main__':
  format_products()
