# -*- coding: utf-8 -*-

import os
import time
import logging
import logging.handlers
import json

import click
from kombu import Connection

from em_celery import logger
from em_celery.tasks.shopify_delete_product_task import delete_shopify_product


@click.command('Send shopify product delete task to worker.')
@click.option('-b', '--broker_url', type=str, required=True, help='Celery worker broker URL.')
@click.option('-q', '--qps', type=float, help='Quantity per second (QPS) to send task.')
@click.option('-s', '--store_code', type=str, required=True, help='Store code that products shall be uploaded to.')
@click.argument('products_path')
def send_shopify_product_delete_task(products_path, store_code, broker_url, qps):
  products_path = os.path.abspath(os.path.expanduser(products_path))
  if not os.path.isfile(products_path):
    logger.error('Could not find products file {}'.format(products_path))
    return

  log_dir = os.path.join(os.path.expanduser('~'), '.em_celery', 'logs')
  if not os.path.isdir(log_dir):
    os.makedirs(log_dir)
  log_path = os.path.join(log_dir, 'shopify_product_delete_task_sender.log')
  level = logging.INFO
  max_bytes = 20 * 1024 ** 2
  fh = logging.handlers.RotatingFileHandler(
    log_path, maxBytes=max_bytes, backupCount=5)
  fh.setLevel(level)
  formatter = logging.Formatter('%(asctime)s %(name)s [%(levelname)s]:%(message)s')
  fh.setFormatter(formatter)
  logger.addHandler(fh)

  last_send_time = None
  connection = Connection(broker_url)
  queue = 'ShopifyProductDelete'
  with open(products_path, encoding='utf-8', errors='ignore') as fh:
    for line in fh:
      product_id = line.strip()
      if not product_id:
        continue

      if last_send_time:
        wait_time = 1 / qps - (time.time() - last_send_time)
        if wait_time > 0:
          logger.debug("Waiting %.3fs to send next message", wait_time)
          time.sleep(wait_time)

      last_send_time = time.time()

      delete_shopify_product.apply_async(
        args=(store_code, product_id),
        queue=queue,
        connection=connection)
      logger.debug(
        'Added delete_shopify_product(%s, %s)',
        store_code, product_id)

if __name__ == "__main__":
  send_shopify_product_delete_task()
